// ----------------------------------------------------------------------
// Overview
//   Camera operation class
// ----------------------------------------------------------------------
// Reference
//   C:\work\CTR\SampleDemos\camera\Simple.cpp/h
// ----------------------------------------------------------------------
#ifndef __CTRL_CAMERA__
#define __CTRL_CAMERA__

// ----------------------------------------------------------------------
// include system header
// ----------------------------------------------------------------------
#include	<nn/camera.h>

// ----------------------------------------------------------------------
// using namespace
// ----------------------------------------------------------------------
using namespace nn::camera;
using namespace nn::y2r;


#include	"TxtWinDebugTimer.h"
// ----------------------------------------------------------------------
// define
// ----------------------------------------------------------------------
// Set Y2R
#define Y2R_INPUT_FORMAT        nn::y2r::INPUT_YUV422_BATCH
#define Y2R_OUTPUT_FORMAT       nn::y2r::OUTPUT_RGB_32
#define Y2R_BLOCK_ALIGNMENT     nn::y2r::BLOCK_LINE
#define Y2R_ROTATION            nn::y2r::ROTATION_NONE
#define YUV_BUFFER_NUM          (3)                  // Number of buffers for transferring (triple buffer)

// Port and camera that are used
#define CAMERA_PORT             nn::camera::PORT_CAM1       // Port 1
#define SELECT_CAMERA           nn::camera::SELECT_IN1      // Inner camera

#define ALIGNMENT_SIZE          (64)
// ----------------------------------------------------------------------
// Setting functions
// ----------------------------------------------------------------------
void	SetExporse				(int n);
void	SetAutoExporse			(int n);
void	SetSharpness			(int n);
void	SetPhotoMode			(int n);
void	SetWhiteBalance			(int n);
void	SetAutoWhiteBalance		(int n);
void	SetContrast				(int n);
void	SetLensCorrection		(int n);

// ----------------------------------------------------------------------
// enum string acquisition functions
// ----------------------------------------------------------------------
void	GetPhotoModeStr			(int n, char* str);
void	GetWhiteBalanceStr		(int n, char* str);
void	GetContrastStr			(int n, char* str);
void	GetLensCorrectionStr	(int n, char* str);

// ----------------------------------------------------------------------
// class
// ----------------------------------------------------------------------
class CCtrlCamera
{
	// Construct/Destruct
public:
	CCtrlCamera();
	~CCtrlCamera();

	int			InitializeCamera	(CameraSelect select, Size size, FrameRate rate);
	void		FinalizeCamera		();

	void		Receive				(u8* rgbBuffer, u8* yuvBuffer, CTxtWinDebugTimer& timer );

    void		CameraThreadFuncImpl();
protected:

private:

    // ----------------------------------------
    // Functions for camera thread control
    void InitializeCameraThread(void);
    void FinalizeCameraThread(void);
//    void PrepareTransitionCallback(void);
//    void AfterTransitionCallback(void);

    // ----------------------------------------
    // Camera/Y2R control functions
    void SetupCamera(void);
    nn::Result InitializeCameraSetting(void);
    void InitializeY2r(void);
    void InitializeResource(void);
    void FinalizeResource(void);
    void CameraRecvFunc(void);
    void CameraVsyncFunc(void);
    void StartCameraCapture(void);
    void StopCameraCapture(void);
    bool Y2rConversion(void);


	u32			m_width;
	u32			m_height;

    CameraSelect m_Select;
    Size         m_Size;
    FrameRate    m_FrameRate;
    NN_PADDING1;

    // -------------------------------------
    // Camera/Y2R transfer
    // -------------------------------------
    // Related to the buffer for transferring YUV data
    s32 s_YuvTransferUnit;  // Transfer size for one time
    s32 s_YuvBufferSize;    // Total size to transfer (size for one frame)
    nn::y2r::StandardCoefficient s_Coefficient;    // The Y2R conversion coefficient suitable to the data output by the camera.
    NN_PADDING3;

    u8* s_pYuvBuffer[YUV_BUFFER_NUM];              // Buffer for transferring data

    // Buffer index
    s32 s_YuvBufferIndexCapturing;      // Buffer to read the image captured by the camera
    s32 s_YuvBufferIndexLatestCaptured; // Buffer where the read from the camera was completed
    s32 s_YuvBufferIndexTransmitting;   // Buffer which is transferring to the Y2R circuit

    nn::os::CriticalSection s_CsYuvSwap;    // For exclusively controlling the Yuv buffer

    // Related to the buffer for transferring RGB data
    s32 s_RgbBufferSize;            // Total size to transfer (size for one frame)
    u8* s_pRgbBuffer;               // Buffer for transferring data

    // The thread that processes camera events
    nn::os::Thread s_CameraThread;
    // Priority of the main thread
    s32 s_MainThreadPriority;

    // -------------------------------------
    // Event
    // -------------------------------------
    // Notification of completed data transfer from the camera to the buffer
    nn::os::Event s_CameraRecvEvent;

    // Can also be used for buffer error notification when sending data from camera to buffer, and when the main thread starts the camera thread.
    nn::os::Event s_CameraBufferErrorEvent;

    // Notification of a camera V blank interrupt with camera capture
    nn::os::Event s_CameraVsyncEvent;

    // Notification of completion of conversion and transfer with Y2R
    nn::os::Event s_Y2rEndEvent;

    // Event to wait for the camera thread to enter the wait state.
    nn::os::LightEvent s_CameraThreadSleepAckEvent;

    // Event to cancel the camera thread wait state.
    nn::os::LightEvent s_CameraThreadAwakeEvent;


    // -------------------------------------
    // Sets flags.
    // -------------------------------------
    // Flag determining whether camera initialization has completed.
    bool s_IsCameraInitialized;

    // Flag to put the camera thread into the wait state.
    bool s_IsCameraThreadSleep;

    // Flag used to stop the camera thread
    bool s_IsCameraThreadEnd;

    // Flag when converting the frame rate
    bool s_SwitchFrameRateFlag;

    // Flag indicating whether there are any images waiting for Y2R conversion.
    bool s_ExistYuvImage;

    NN_PADDING3;


};

#endif	// __CTRL_CAMERA__
