// ----------------------------------------------------------------------
// Text menu class
// ----------------------------------------------------------------------

// ----------------------------------------------------------------------
// include system header
// ----------------------------------------------------------------------
#include	<nn.h>
#include	<nn/gx.h>
#include	<nn/math.h>
#include	<nn/os.h>
#include	<nn/fnd.h>
#include	<nn/fs.h>

// ----------------------------------------------------------------------
// include original header
// ----------------------------------------------------------------------
#include	"CTR_Define.h"
#include	"TxtMenu.h"

// ----------------------------------------------------------------------
// CTxtMenu::CTxtMenu
// ----------------------------------------------------------------------
// Content
//   Constructor
// ----------------------------------------------------------------------
// Arguments
//   maxItem		[ I ] Specifies number of menus.
//   title			[ I ] Specifies WindowTitle string.
// ----------------------------------------------------------------------
// Return value
//   None
// ----------------------------------------------------------------------
CTxtMenu::CTxtMenu(int maxItem, char* title)
{
	m_maxItem	= maxItem,
	m_currItem	= 0;
	m_depth		= 0;
	m_item_info	= (LPITEM_INFO)new ITEM_INFO[m_maxItem];
	sprintf(m_title, title);

	m_menu		= this;
	m_parent	= NULL;
}

// ----------------------------------------------------------------------
// CTxtMenu::~CTxtMenu
// ----------------------------------------------------------------------
// Content
//   Destructor
// ----------------------------------------------------------------------
// Arguments
//   None
// ----------------------------------------------------------------------
// Return value
//   None
// ----------------------------------------------------------------------
CTxtMenu::~CTxtMenu()
{
	if (m_item_info != NULL)
	{
		delete [] m_item_info;
		m_item_info = NULL;
	}
}

// ----------------------------------------------------------------------
// CTxtMenu::SetParent
// ----------------------------------------------------------------------
// Content
//   Sets parent window.
//   When selecting Exit in current menu, current position shifts to the parent.
// ----------------------------------------------------------------------
// Arguments
//   parent			[ I ] Specifies parent menu.
// ----------------------------------------------------------------------
// Return value
//   None
// ----------------------------------------------------------------------
void CTxtMenu::SetParent(CTxtMenu* parent)
{
	m_parent = parent;
}

// ----------------------------------------------------------------------
// CTxtMenu::SetItemOnOff
// ----------------------------------------------------------------------
// Content
//   Configures item in bool format.
//   Similar to a checkbox
// ----------------------------------------------------------------------
// Arguments
//   no			[ I ] Specifies set item number.
//   nID		[ I ] Specifies set item ID.
//   str		[ I ] Specifies display string.
//   bDefault	[ I ] Specifies default value.
//   func		[ I ] Specifies pointer to function to call when value is changed.
// ----------------------------------------------------------------------
// Return value
//   None
// ----------------------------------------------------------------------
void CTxtMenu::SetItemOnOff(int no, int nID, char* str, bool bDefault, FUNC func)
{
	LPITEM_INFO info = &m_item_info[no];
	info->id		= nID;
	info->type		= eITEM_ONOFF;

	info->bOnOff	= bDefault;
	info->min		= 0;
	info->max		= 0;
	info->nDefault	= 0;
	sprintf(info->str, str);
	info->child		= NULL;
	info->func		= func;
	info->getStr	= NULL;
	info->add		= 0;
}

// ----------------------------------------------------------------------
// CTxtMenu::SetItemMinMax
// ----------------------------------------------------------------------
// Content
//   Configures item in int format.
//   Specifies range in continuous integer values.
// ----------------------------------------------------------------------
// Arguments
//   no			[ I ] Specifies item number to set.
//   nID		[ I ] Specifies item ID to set.
//   str		[ I ] Specifies display string.
//   min		[ I ] Specifies minimum value.
//   max		[ I ] Specifies maximum value.
//   nDefault	[ I ] Specifies default value.
//   func		[ I ] Specifies pointer to function to call when value is changed.
//   add		[ I ] Specifies amount of fluctuation.
// ----------------------------------------------------------------------
// Return value
//   None
// ----------------------------------------------------------------------
// 2010.12.01 Added add
// ----------------------------------------------------------------------
void CTxtMenu::SetItemMinMax(int no, int nID, char* str, int min, int max, int nDefault, FUNC func, int add)
{
	LPITEM_INFO info = &m_item_info[no];
	info->id		= nID;
	info->type		= eITEM_MINMAX;

	info->bOnOff	= false;
	info->min		= min;
	info->max		= max;
	info->nDefault	= nDefault;
	sprintf(info->str, str);
	info->child		= NULL;
	info->func		= func;
	info->getStr	= NULL;
	info->add		= add;
}

// ----------------------------------------------------------------------
// CTxtMenu::SetSubMenu
// ----------------------------------------------------------------------
// Content
//   Configures submenu.
// ----------------------------------------------------------------------
// Arguments
//   no			[ I ] Specifies item number to set.
//   nID		[ I ] Specifies item ID to set.
//   str		[ I ] Specifies display string.
//   menu		[ I ] Specifies submenu.
//   func		[ I ] Specifies pointer to function to call when value is changed.
// ----------------------------------------------------------------------
// Return value
//   None
// ----------------------------------------------------------------------
void CTxtMenu::SetSubMenu(int no, int nID, char* str, CTxtMenu* menu, FUNC func)
{
	LPITEM_INFO info = &m_item_info[no];
	info->id		= nID;
	info->type		= eITEM_SUB_MENU;

	info->bOnOff	= false;
	info->min		= 0;
	info->max		= 0;
	info->nDefault	= 0;
	sprintf(info->str, str);
	info->child		= (void*)menu;
	info->func		= func;
	info->getStr	= NULL;
	info->add		= 0;

	menu->SetParent(this);
}

// ----------------------------------------------------------------------
// CTxtMenu::SetItemExit
// ----------------------------------------------------------------------
// Content
//   Configures "Return" menu.
// ----------------------------------------------------------------------
// Arguments
//   no			[ I ] Specifies item number to set.
//   nID		[ I ] Specifies item ID to set.
//   str		[ I ] Specifies display string.
//   func		[ I ] Specifies pointer to function to call when value is changed.
// ----------------------------------------------------------------------
// Return value
//   None
// ----------------------------------------------------------------------
void CTxtMenu::SetItemExit(int no, int nID, char* str, FUNC func)
{
	LPITEM_INFO info = &m_item_info[no];
	info->id		= nID;
	info->type		= eITEM_EXIT;

	info->bOnOff	= false;
	info->min		= 0;
	info->max		= 0;
	info->nDefault	= 0;
	sprintf(info->str, str);
	info->child		= NULL;
	info->func		= func;
	info->getStr	= NULL;
	info->add		= 0;
}

// ----------------------------------------------------------------------
// CTxtMenu::SetItemStrFunc
// ----------------------------------------------------------------------
// Content
//   Specifies pointer to function that gets value string to display for minmax.
//   Example: Used when displaying MIN, MID, MAX, etc. when the value is 0 - 2 with enum.
// ----------------------------------------------------------------------
// Arguments
//   nID		[ I ] Specifies item ID to set.
//   getStr		[ I ] Specifies pointer to function to call when value is changed.
// ----------------------------------------------------------------------
// Return value
//   None
// ----------------------------------------------------------------------
void CTxtMenu::SetItemStrFunc(int nID, FUNC_STR getStr)
{
	for (int n = 0 ; n < m_maxItem ; n++)
	{
		LPITEM_INFO info = &m_item_info[n];
		if (info->id != nID)
		{
			continue;
		}
		info->getStr = getStr;
	}
	return;
}

// ----------------------------------------------------------------------
// CTxtMenu::GetCount
// ----------------------------------------------------------------------
// Content
//   Gets number of currently enabled menu items.
// ----------------------------------------------------------------------
// Arguments
//   None
// ----------------------------------------------------------------------
// Return value
//   Returns number of items.
// ----------------------------------------------------------------------
int CTxtMenu::GetCount()
{
	if (m_menu != this)
	{
		return m_menu->GetCount();
	}
	return m_maxItem;
}

// ----------------------------------------------------------------------
// CTxtMenu::GetTitle
// ----------------------------------------------------------------------
// Content
//   Gets number of currently enabled menu title strings.
// ----------------------------------------------------------------------
// Arguments
//   str		[ O ] Stores acquired title string.
// ----------------------------------------------------------------------
// Return value
//   None
// ----------------------------------------------------------------------
void CTxtMenu::GetTitle(char* str)
{
	if (m_menu != this)
	{
		m_menu->GetTitle(str);
		return;
	}
	sprintf(str, m_title);
}

// ----------------------------------------------------------------------
// CTxtMenu::GetItemText
// ----------------------------------------------------------------------
// Content
//   Gets specified currently enabled menu item.
// ----------------------------------------------------------------------
// Arguments
//   no			[ I ] Specifies item number to set.
//   str		[ O ] Stores acquired item string.
// ----------------------------------------------------------------------
// Return value
//   None
// ----------------------------------------------------------------------
void CTxtMenu::GetItemText(int no, char* str)
{
	if (m_menu != this)
	{
		m_menu->GetItemText(no, str);
		return;
	}

	LPITEM_INFO info = &m_item_info[no];

	char str_select[8];
	if (no == m_currItem)
	{
		sprintf(str_select, "[>]");
	}
	else
	{
		sprintf(str_select, "[ ]");
	}

	char str_subItem[8];
	if (info->type == eITEM_SUB_MENU)
	{
		sprintf(str_subItem, "@");
	}
	else
	{
		sprintf(str_subItem, " ");
	}

	char str_ret[16];

	switch(info->type)
	{
	case eITEM_ONOFF:
		if (info->bOnOff == true)
		{
			sprintf(str_ret, "ON");
		}
		else
		{
			sprintf(str_ret, "OFF");
		}
		sprintf(str, "%s%s%s(ON/OFF) :%s", str_select, str_subItem, info->str, str_ret);
		break;

	case eITEM_MINMAX:
		if (info->getStr == NULL)
		{
			sprintf(str, "%s%s%s(%2d ~ %2d):%d", str_select, str_subItem, info->str, info->min, info->max, info->nDefault);
		}
		else
		{
			(info->getStr)(info->nDefault, str_ret);
			sprintf(str, "%s%s%s         :%s", str_select, str_subItem, info->str, str_ret);
		}
		break;

	case eITEM_SUB_MENU:
		sprintf(str, "%s%s%s", str_select, str_subItem, info->str);
		break;

	case eITEM_EXIT:
		sprintf(str, "%s%s%s", str_select, str_subItem, info->str);
		break;
	}
}

// ----------------------------------------------------------------------
// CTxtMenu::SetNext
// ----------------------------------------------------------------------
// Content
//   Moves cursor position in currently enabled menu to next position.
//   Note: Returns to top when moving down when already at bottom.
// ----------------------------------------------------------------------
// Arguments
//   None
// ----------------------------------------------------------------------
// Return value
//   None
// ----------------------------------------------------------------------
void CTxtMenu::SetNext()
{
	if (m_menu != this)
	{
		m_menu->SetNext();
		return;
	}

	m_currItem++;
	if (m_currItem >= m_maxItem)
	{
		m_currItem = 0;
	}
}

// ----------------------------------------------------------------------
// CTxtMenu::SetPrev
// ----------------------------------------------------------------------
// Content
//   Moves cursor position in currently enabled menu to previous position.
//   Note: Moves to bottom when moving up when already at top.
// ----------------------------------------------------------------------
// Arguments
//   None
// ----------------------------------------------------------------------
// Return value
//   None
// ----------------------------------------------------------------------
void CTxtMenu::SetPrev()
{
	if (m_menu != this)
	{
		m_menu->SetPrev();
		return;
	}

	m_currItem--;
	if (m_currItem < 0)
	{
		m_currItem = m_maxItem - 1;
	}
}

// ----------------------------------------------------------------------
// CTxtMenu::SetSelect
// ----------------------------------------------------------------------
// Content
//   Executes item at cursor position in currently enabled menu.
// ----------------------------------------------------------------------
// Arguments
//   bUp		[ I ] When true, increases the value; when false, decreases the value.
//                    
// ----------------------------------------------------------------------
// Return value
//   None
// ----------------------------------------------------------------------
void CTxtMenu::SetSelect(bool bUp)
{
	m_menu = SetChiledSelect(bUp);
}

// ----------------------------------------------------------------------
// CTxtMenu::SetChildSelect
// ----------------------------------------------------------------------
// Content
//   Executes item at cursor position in currently enabled menu.
// ----------------------------------------------------------------------
// Arguments
//   bUp		[ I ] When true, increases the value; when false, decreases the value.
//                    
// ----------------------------------------------------------------------
// Return value
//   None
// ----------------------------------------------------------------------
CTxtMenu* CTxtMenu::SetChiledSelect(bool bUp)
{
	if (m_menu != this)
	{
		m_menu = m_menu->SetChiledSelect(bUp);
		return m_menu;
	}

	LPITEM_INFO info = &m_item_info[m_currItem];
	if (info->type == eITEM_SUB_MENU)
	{
		return (CTxtMenu*)info->child;
	}
	else if (info->type == eITEM_EXIT)
	{
		if (info->func != NULL)
		{
			(info->func)(m_parent != NULL);
		}
		if (m_parent != NULL)
		{
			return m_parent;
		}
		return this;
	}
	else if (info->type == eITEM_ONOFF)
	{
		info->bOnOff = (info->bOnOff == true) ? false : true;
		if (info->func != NULL)
		{
			(info->func)(info->bOnOff);
		}
	}
	else if (info->type == eITEM_MINMAX)
	{
		info->nDefault += (bUp == true) ? info->add : (info->add * -1);
		info->nDefault = (info->nDefault < info->min) ? info->max : info->nDefault;
		info->nDefault = (info->nDefault > info->max) ? info->min : info->nDefault;
		if (info->func != NULL)
		{
			(info->func)(info->nDefault);
		}
	}
	return this;
}

// ----------------------------------------------------------------------
// CTxtMenu::GetItemOnOff
// ----------------------------------------------------------------------
// Content
//   Gets value for specified ID.
// ----------------------------------------------------------------------
// Arguments
//   nID		[ I ] Specifies item ID to set.
// ----------------------------------------------------------------------
// Return value
//   Returns bool value.
// ----------------------------------------------------------------------
bool CTxtMenu::GetItemOnOff(int nID)
{
	for (int n = 0 ; n < m_maxItem ; n++)
	{
		LPITEM_INFO info = &m_item_info[n];
		if (info->id != nID)
		{
			continue;
		}
		return info->bOnOff;
	}
	return false;
}

// ----------------------------------------------------------------------
// CTxtMenu::GetItemMinMax
// ----------------------------------------------------------------------
// Content
//   Gets value for specified ID.
// ----------------------------------------------------------------------
// Arguments
//   nID		[ I ] Specifies item ID to set.
// ----------------------------------------------------------------------
// Return value
//   Returns int value.
// ----------------------------------------------------------------------
int CTxtMenu::GetItemMinMax(int nID)
{
	for (int n = 0 ; n < m_maxItem ; n++)
	{
		LPITEM_INFO info = &m_item_info[n];
		if (info->id != nID)
		{
			continue;
		}
		return info->nDefault;
	}
	return 0;
}
