// ----------------------------------------------------------------------
// Text Window Class
// ----------------------------------------------------------------------

// ----------------------------------------------------------------------
// include system header
// ----------------------------------------------------------------------
#include	<nn.h>
#include	<nn/gx.h>
#include	<nn/math.h>
#include	<nn/os.h>
#include	<nn/fnd.h>
#include	<nn/fs.h>

#include	<string.h>

// ----------------------------------------------------------------------
// Includes
// ----------------------------------------------------------------------
#include	"CTR_Define.h"
#include	"CtrlGraphics.h"
#include	"TxtWindow.h"

// ----------------------------------------------------------------------
// define
// ----------------------------------------------------------------------
#define		TITLE_HEIGHT	10
#define		FONT_WIDTH		8
#define		FONT_HEIGHT		8

// ----------------------------------------------------------------------
// CTxtWindow::CTxtWindow
// ----------------------------------------------------------------------
// Content
//   Constructor
// ----------------------------------------------------------------------
// Arguments
//   title		[ I ] Specifies WindowTitle string.
// ----------------------------------------------------------------------
// Return value
//   None
// ----------------------------------------------------------------------
CTxtWindow::CTxtWindow(char* title)
{
	m_info = new STR_INFO[20];
	for (int n = 0 ; n < 20 ; n++)
	{
		LPSTR_INFO info = &m_info[n];
		info->str[0] = '\0';
		info->r		 = 0.0f;
		info->g		 = 0.0f;
		info->b		 = 0.0f;
	}
	sprintf(m_title, "%s", title);
	m_line		= 0;
	m_width		= 0;
	m_height	= 0;
	m_bForcus	= false;
}

// ----------------------------------------------------------------------
// CTxtWindow::~CTxtWindow
// ----------------------------------------------------------------------
// Content
//   Destructor
// ----------------------------------------------------------------------
// Arguments
//   None
// ----------------------------------------------------------------------
// Return value
//   None
// ----------------------------------------------------------------------
CTxtWindow::~CTxtWindow()
{
	if (m_info != NULL)
	{
		delete [] m_info;
		m_info = NULL;
	}
}

// ----------------------------------------------------------------------
// CTxtWindow::SetLines
// ----------------------------------------------------------------------
// Content
//   Specifies number of lines to display. (maximum 20 lines)
// ----------------------------------------------------------------------
// Arguments
//   line		[ I ] Specifies number of lines.
// ----------------------------------------------------------------------
// Return value
//   None
// ----------------------------------------------------------------------
void CTxtWindow::SetLines(int line)
{
	if (line > 20) line = 20;
	m_line = line;
}

// ----------------------------------------------------------------------
// CTxtWindow::SetText
// ----------------------------------------------------------------------
// Content
//   Specifies string to display.
// ----------------------------------------------------------------------
// Arguments
//   line		[ I ] Specifies line number to set.
//   str		[ I ] Specifies string.
//   r			[ I ] Specifies red (0.0f - 1.0f).
//   g			[ I ] Specifies green (0.0f - 1.0f).
//   b			[ I ] Specifies blue (0.0f - 1.0f).
// ----------------------------------------------------------------------
// Return value
//   None
// ----------------------------------------------------------------------
void CTxtWindow::SetText(int line, char* str, GLfloat r, GLfloat g, GLfloat b)
{
	LPSTR_INFO info = &m_info[line];
	sprintf(info->str, str);
	info->r = r;
	info->g = g;
	info->b = b;
}

// ----------------------------------------------------------------------
// CTxtWindow::DrawWindow
// ----------------------------------------------------------------------
// Content
//   Displays Window.
// ----------------------------------------------------------------------
// Arguments
//   graphics		[ I ] Specifies render class. 
//   display		[ I ] Specifies display to render.
//   sx				[ I ] Specifies x-coordinate to start rendering.
//   sy				[ I ] Specifies y-coordinate to end rendering.
// ----------------------------------------------------------------------
// Return value
//   None
// ----------------------------------------------------------------------
void CTxtWindow::DrawWindow(CCtrlGraphics* graphics, const s32 display, int sx, int sy)
{
	int width	= m_width;
	int height	= m_height;

	int text_width	= 0;
	int text_height = 0;

	// ----------------------------------------------------------------------
	// Determine width from longest string
	// ----------------------------------------------------------------------
	int max_len = 0;
	int n = 0;
	for (n = 0 ; n < m_line ; n++)
	{
		LPSTR_INFO info = &m_info[n];
		max_len = (strlen(info->str) > max_len) ? strlen(info->str) : max_len;
	}
	text_width  = max_len * FONT_WIDTH + 2;

	// ----------------------------------------------------------------------
	// Determine height from number of lines
	// ----------------------------------------------------------------------
	text_height = TITLE_HEIGHT + m_line * FONT_HEIGHT + 2;

	width  = (m_width  > text_width ) ? m_width  : text_width;
	height = (m_height > text_height) ? m_height : text_height;

	// Centering
	if (sx < 0)
	{
		sx = (320 - width) / 2;
	}

	if (sy < 0)
	{
		sy = (240 - height) / 2;
	}

	// ----------------------------------------------------------------------
	// Render title portion dark blue/light blue field (switches with focus)
	// ----------------------------------------------------------------------
	int rect_sx = sx;
	int rect_ex = sx + width;
	int rect_sy = sy;
	int rect_ey = sy + height;

	if (m_bForcus == true)
	{
		graphics->SetColor(display, 0.0f, 0.0f, 0.5f);
		graphics->DrawFull(display, rect_sx, rect_sy, rect_ex, rect_ey);
	}
	else
	{
		graphics->SetColor(display, 0.5f, 0.7f, 1.0f);
		graphics->DrawFull(display, rect_sx, rect_sy, rect_ex, rect_ey);
	}

	// ----------------------------------------------------------------------
	// Display Window portion gray
	// ----------------------------------------------------------------------
	rect_sx = sx + 1;
	rect_ex = sx + width - 1;
	rect_sy = sy + TITLE_HEIGHT;
	rect_ey = sy + height - 1;

	graphics->SetColor(display, 0.5f, 0.5f, 0.5f);
	graphics->DrawFull(display, rect_sx, rect_sy, rect_ex, rect_ey);

	// ----------------------------------------------------------------------
	// Render title string (white letters)
	// ----------------------------------------------------------------------
	int txt_sx  = sx + 1;
	int txt_sy  = sy + 1;
	graphics->SetColor(display, 1.0f, 1.0f, 1.0f);
	graphics->DrawTextEx(display, txt_sx, txt_sy, m_title);

	// ----------------------------------------------------------------------
	// Renders string collection (with each set color)
	// ----------------------------------------------------------------------
	txt_sx  = sx + 1;
	txt_sy  = txt_sy + TITLE_HEIGHT;
	for (n = 0 ; n < m_line ; n++)
	{
		LPSTR_INFO info = &m_info[n];
		graphics->SetColor(display, info->r, info->g, info->b);
		graphics->DrawTextEx(display, txt_sx + 1, txt_sy + FONT_HEIGHT * n, info->str);
	}
}

// ----------------------------------------------------------------------
// CTxtWindow::SetWindowWidth
// ----------------------------------------------------------------------
// Content
//   Specifies Window width.
//   When 0 is specified, automatically adjusts to match displayed string.
// ----------------------------------------------------------------------
// Arguments
//   width			[ I ] Specifies Window width.
// ----------------------------------------------------------------------
// Return value
//   None
// ----------------------------------------------------------------------
void CTxtWindow::SetWindowWidth(int width)
{
	m_width = width;
}

// ----------------------------------------------------------------------
// CTxtWindow::SetWindowHeight
// ----------------------------------------------------------------------
// Content
//   Specifies Window height.
//   When 0 is specified, automatically adjusts to match displayed string.
// ----------------------------------------------------------------------
// Arguments
//   height			[ I ] Specifies Window height.
// ----------------------------------------------------------------------
// Return value
//   None
// ----------------------------------------------------------------------
void CTxtWindow::SetWindowHeight(int height)
{
	m_height = height;
}

// ----------------------------------------------------------------------
// CTxtWindow::SetFocus
// ----------------------------------------------------------------------
// Content
//   Configures focus.
// ----------------------------------------------------------------------
// Arguments
//   bForcus		[ I ] Specify true to reach focus.
// ----------------------------------------------------------------------
// Return value
//   None
// ----------------------------------------------------------------------
void CTxtWindow::SetForcus(bool bForcus)
{
	m_bForcus = bForcus;
}
