// ----------------------------------------------------------------------
// Timer display Window class for debugging
// ----------------------------------------------------------------------
// Main content
// - Displays speed
// - Display in red when exceeding 10% of the whole
// ----------------------------------------------------------------------

// ----------------------------------------------------------------------
// include system header
// ----------------------------------------------------------------------
#include	<nn.h>
#include	<nn/gx.h>
#include	<nn/math.h>
#include	<nn/os.h>
#include	<nn/hid.h>
#include	<nn/fnd.h>
#include	<nn/fs.h>

#include	<string.h>

// ----------------------------------------------------------------------
// include original header
// ----------------------------------------------------------------------
#include	"CTR_Define.h"
#include	"CtrlGraphics.h"
#include	"TxtWindow.h"
#include	"TxtWinDebugTimer.h"

// ----------------------------------------------------------------------
//CTxtWinDebugTimer::CTxtWinDebugTimer
// ----------------------------------------------------------------------
// Content
//   Constructor
// ----------------------------------------------------------------------
// Arguments
//   max_timer	[ I ] Specifies the number of timers.
// ----------------------------------------------------------------------
// Return value.
//   None
// ----------------------------------------------------------------------
CTxtWinDebugTimer::CTxtWinDebugTimer(int max_timer)
	: CTxtWindow("Debug Timer")
{
	int n = 0;
	for (n = 0 ; n < MAX_TIMER ; n++)
	{
		sprintf(m_str[n], "---");
		m_time_st[n] = 0;
		m_time[n]	 = 0;
	}

	m_max_timer = max_timer;
}

// ----------------------------------------------------------------------
// CTxtWinDebugTimer::~CTxtWinDebugTimer
// ----------------------------------------------------------------------
// Content
//   Destructor
// ----------------------------------------------------------------------
// Arguments
//   None
// ----------------------------------------------------------------------
// Return value.
//   None
// ----------------------------------------------------------------------
CTxtWinDebugTimer::~CTxtWinDebugTimer()
{
}

// ----------------------------------------------------------------------
// CTxtWinDebugTimer::Draw
// ----------------------------------------------------------------------
// Content
//   Displays Window.
// ----------------------------------------------------------------------
// Arguments
//   graphics		[ I ] Specifies the render class. 
//   display		[ I ] Specifies the display to render.
//   sx				[ I ] Specifies the x-coordinate to start rendering.
//   sy				[ I ] Specifies the y-coordinate to end rendering.
// ----------------------------------------------------------------------
// Return value.
//   Always returns 0.
// ----------------------------------------------------------------------
int CTxtWinDebugTimer::Draw(CCtrlGraphics* graphics, const s32 display, int sx, int sy)
{
	char	str[128];
	int		n	= 0;
	unsigned long	sum = 0;
	float	per = 0.0f;

	for (n = 0 ; n < 9 ;n++)
	{
		sum+= m_time[n];
	}

	CTxtWindow::SetLines(12);

#if defined( TIME_SET_MSEC )
	sprintf(str, "title              [   ms]:per\n");
#else
	sprintf(str, "title              [   us]:per\n");
#endif
	CTxtWindow::SetText(0, str, 0.0, 0.0, 0.0);

	sprintf(str, "------------------------------\n");
	CTxtWindow::SetText(1, str, 1.0, 1.0, 1.0);

	for (n = 0 ; n < m_max_timer ; n++)
	{
		per = (float)m_time[n] / (float)sum * 100.0f;
		sprintf(str, "%02d:%s [%5d]:%4.2f\n", n + 1, m_str[n], m_time[n], per);

		if (per > 10.0f)
		{
			// Red display when 10% is exceeded
			CTxtWindow::SetText(n + 2, str, 1.0, 0.0, 0.0);
		}
		else
		{
			// Otherwise display in black.
			CTxtWindow::SetText(n + 2, str, 0.0, 0.0, 0.0);
		}
	}

	// Separator
	sprintf(str, "------------------------------\n");
	CTxtWindow::SetText(n + 2, str, 1.0, 1.0, 1.0);

	// Display total
	sprintf(str, "All                [%5d]\n", m_time_all);
	CTxtWindow::SetText(n + 3, str, 0.0, 0.0, 0.0);

	CTxtWindow::DrawWindow(graphics, display, sx, sy);
	return 0;
}

// ----------------------------------------------------------------------
// CTxtWinDebugTimer::SetTimerText
// ----------------------------------------------------------------------
// Content
//   Sets the item title.
// ----------------------------------------------------------------------
// Arguments
//   No			[ I ] Specifies the number.
//   str		[ I ] Specifies the title string.
// ----------------------------------------------------------------------
// Return value.
//   None
// ----------------------------------------------------------------------
void CTxtWinDebugTimer::SetTimerText(int no, char* str)
{
	if (no < 0 || no >= m_max_timer)
	{
		return;
	}
	sprintf(m_str[no], str);
}

// ----------------------------------------------------------------------
// CTxtWinDebugTimer::TimerStart
// ----------------------------------------------------------------------
// Content
//   Starts the timer with the specified number.
// ----------------------------------------------------------------------
// Arguments
//   No			[ I ] Specifies the number.
// ----------------------------------------------------------------------
// Return value.
//   None
// ----------------------------------------------------------------------
void CTxtWinDebugTimer::TimerStart(int no)
{
	if (no < 0 || no >= m_max_timer)
	{
		return;
	}
	m_time_st[no] = CTR_GetTick();
}

// ----------------------------------------------------------------------
// CTxtWinDebugTimer::TimerEnd
// ----------------------------------------------------------------------
// Content
//   Terminates the timer with the specified number.
// ----------------------------------------------------------------------
// Arguments
//   No			[ I ] Specifies the number.
// ----------------------------------------------------------------------
// Return value.
//   None
// ----------------------------------------------------------------------
void CTxtWinDebugTimer::TimerEnd(int no)
{
	if (no < 0 || no >= m_max_timer)
	{
		return;
	}
	m_time[no] = CTR_GetTick() - m_time_st[no];
}

// ----------------------------------------------------------------------
// CTxtWinDebugTimer::TimerAllStart
// ----------------------------------------------------------------------
// Content
//   Start the only total timer.
// ----------------------------------------------------------------------
// Arguments
//   None
// ----------------------------------------------------------------------
// Return value.
//   None
// ----------------------------------------------------------------------
void CTxtWinDebugTimer::TimerAllStart()
{
	m_time_all_st = CTR_GetTick();
}

// ----------------------------------------------------------------------
// CTxtWinDebugTimer::TimerAllEnd
// ----------------------------------------------------------------------
// Content
//   Ends the only total timer.
// ----------------------------------------------------------------------
// Arguments
//   None
// ----------------------------------------------------------------------
// Return value.
//   None
// ----------------------------------------------------------------------
void CTxtWinDebugTimer::TimerAllEnd()
{
	m_time_all = CTR_GetTick() - m_time_all_st;
}
